@@ -216,6 +216,8 @@ f '/>/' 'respond' = f
 (f '/>/' g) '/>/' h = f '/>/' (g '/>/' h)
 @
 
+#respond-diagram#
+
     The following diagrams show the flow of information:
 
 @
@@ -231,7 +233,7 @@ f '/>/' 'respond' = f
  x  ==>   / \\===> a
      |    |    |
      +----|----+
-          v 
+          v
           a'
 
 ('/>/') :: 'Monad' m
@@ -250,6 +252,23 @@ f '/>/' 'respond' = f
      +----|----+            \\  +----|----+            +----|----+
           v                  \\      v                      v
           a'                  \\==== b'                     a'
+
+('//>') :: 'Monad' m
+      => 'Proxy' x' x b' b m a'
+      -> (b -> 'Proxy' x' x c' c m b')
+      -> 'Proxy' x' x c' c m a'
+
+\                              /===> b
+                             /      |
+     +---------+            /  +----|----+            +---------+
+     |         |           /   |    v    |            |         |
+ x' <==       <== b' <==\\ / x'<==       <== c'    x' <==       <== c'
+     |    f    |         X     |    g    |     =      | f '//>' g |
+ x  ==>       ==> b  ===/ \\ x ==>       ==> c     x  ==>       ==> c'
+     |    |    |           \\   |    |    |            |    |    |
+     +----|----+            \\  +----|----+            +----|----+
+          v                  \\      v                      v
+          a'                  \\==== b'                     a'
 @
 
 -}
@@ -331,6 +350,8 @@ f '\>\' 'request' = f
 (f '\>\' g) '\>\' h = f '\>\' (g '\>\' h)
 @
 
+#request-diagram#
+
     The following diagrams show the flow of information:
 
 @
@@ -365,6 +386,23 @@ f '\>\' 'request' = f
      +----|----+    /         +----|----+            +----|----+
           v        /               v                      v
           b ======/                c                      c
+
+('>\\') :: Monad m
+      => (b' -> Proxy a' a y' y m b)
+      -> Proxy b' b y' y m c
+      -> Proxy a' a y' y m c
+
+\          b'<=====\\
+          |        \\
+     +----|----+    \\         +---------+            +---------+
+     |    v    |     \\        |         |            |         |
+ a' <==       <== y'  \\== b' <==       <== y'    a' <==       <== y'
+     |    f    |              |    g    |     =      | f '>\\' g |
+ a  ==>       ==> y   /=> b  ==>       ==> y     a  ==>       ==> y
+     |    |    |     /        |    |    |            |    |    |
+     +----|----+    /         +----|----+            +----|----+
+          v        /               v                      v
+          b ======/                c                      c
 @
 -}
 
@@ -552,6 +590,8 @@ f '>+>' 'pull' = f
 (f '>+>' g) '>+>' h = f '>+>' (g '>+>' h)
 @
 
+#pull-diagram#
+
     The following diagrams show the flow of information:
 
 @
